#ifndef         _CTHREAD_BASE_HPP
#define         _CTHREAD_BASE_HPP


#include <pthread.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

//====================================================================
class   CMutexLock
{
        private :
                pthread_mutex_t         pLock;
                void    Init(void)                      { pthread_mutex_init    (&pLock, NULL); }
                
                
        public :
                CMutexLock        (void)                { Init(); }
                ~CMutexLock       (void)                { pthread_mutex_destroy (&pLock); }
                void    Lock  (void)            { pthread_mutex_lock    (&pLock); }
                void    UnLock(void)            { pthread_mutex_unlock  (&pLock); }
};

class   CThreadBase
{
        private :       
                pthread_t               pTID;
                void                    *pArg;
                int             nRet;
        
                pthread_cond_t  gthead_condition;
                pthread_mutex_t gthread_mlock;
        protected :     
                bool                    fgSUSPEND;
                
                pthread_t               GetTid(void){ return pTID; }
                static void*    EntryPoint(void *obj)
                {
                                                CThreadBase             *This = (CThreadBase*)obj;
                                                return This->ThreadRun(This->pArg);
                }

        public :

                CThreadBase(void)
                {       
            		fgSUSPEND=false;
                    pthread_cond_init (&gthead_condition, NULL);
            		pthread_mutex_init(&gthread_mlock,    NULL);
                }
                virtual ~CThreadBase(void)
				{
					pthread_mutex_destroy(&gthread_mlock);
					pthread_cond_destroy(&gthead_condition);
					pthread_detach(pTID);
				}
                virtual void *ThreadRun (void *arg=NULL) = 0;
                
                int  Create     (void *arg=NULL)        { pArg=arg; return pthread_create(&pTID, NULL, EntryPoint, this); }
                int  Stop       (void)                          { return pthread_cancel  (pTID); }
                int  Join       (void **ptr)            { return pthread_join    (pTID, ptr); }
                int  Suspend(void)
                {
            		pthread_mutex_lock(&gthread_mlock);
                    fgSUSPEND=true;
            		nRet = pthread_cond_wait(&gthead_condition, &gthread_mlock);
            		pthread_mutex_unlock(&gthread_mlock);
                    return nRet;
                }
                int  Resume (void)
                {
			        pthread_mutex_lock(&gthread_mlock);
			        nRet = pthread_cond_signal(&gthead_condition);
                    fgSUSPEND=false;
            		pthread_mutex_unlock(&gthread_mlock);
            		return nRet;
                }
                bool GetSuspend(void)                   { return fgSUSPEND; }
                void SetSuspendFlag(bool flag)  { fgSUSPEND=flag;       }

                pthread_mutex_t* GetMutex()     {return &gthread_mlock;}
};

#endif
